import functools
from functools import wraps
from typing import Optional, Callable, Union

from cpl_core.dependency_injection import ServiceProviderABC
from cpl_core.environment import ApplicationEnvironmentABC
from flask import request, jsonify
from flask_cors import cross_origin

from bot_api.abc.auth_service_abc import AuthServiceABC
from bot_api.exception.service_error_code_enum import ServiceErrorCode
from bot_api.exception.service_exception import ServiceException
from bot_api.model.error_dto import ErrorDTO
from bot_data.abc.auth_user_repository_abc import AuthUserRepositoryABC
from bot_data.model.auth_role_enum import AuthRoleEnum
from bot_data.model.auth_user import AuthUser


class Route:
    registered_routes = {}

    _auth_users: Optional[AuthUserRepositoryABC] = None
    _auth: Optional[AuthServiceABC] = None
    _env = "production"

    @classmethod
    @ServiceProviderABC.inject
    def init_authorize(
        cls,
        env: ApplicationEnvironmentABC,
        auth_users: AuthUserRepositoryABC,
        auth: AuthServiceABC,
    ):
        cls._auth_users = auth_users
        cls._auth = auth
        cls._env = env.environment_name

    @classmethod
    def get_user(cls) -> Optional[Union[str, AuthUser]]:
        token = None
        api_key = None
        authorization = request.headers.get("Authorization").split()
        match authorization[0]:
            case "Bearer":
                token = authorization[1]
            case "API-Key":
                api_key = authorization[1]

        if api_key is not None:
            return "system"

        if token is None:
            return None

        jwt = cls._auth.decode_token(token)
        user = cls._auth_users.get_auth_user_by_email(jwt["email"])
        return user

    @classmethod
    def authorize(
        cls,
        f: Callable = None,
        role: AuthRoleEnum = None,
        skip_in_dev=False,
        by_api_key=False,
    ):
        if f is None:
            return functools.partial(cls.authorize, role=role, skip_in_dev=skip_in_dev, by_api_key=by_api_key)

        @wraps(f)
        async def decorator(*args, **kwargs):
            if skip_in_dev and cls._env == "development":
                return await f(*args, **kwargs)

            token = None
            api_key = None
            if "Authorization" in request.headers:
                if " " not in request.headers.get("Authorization"):
                    ex = ServiceException(ServiceErrorCode.Unauthorized, f"Token not set")
                    error = ErrorDTO(ex.error_code, ex.message)
                    return jsonify(error.to_dict()), 401

                authorization = request.headers.get("Authorization").split()
                match authorization[0]:
                    case "Bearer":
                        token = authorization[1]
                    case "API-Key":
                        api_key = authorization[1]

            if api_key is not None:
                valid = False
                try:
                    valid = cls._auth.verify_api_key(api_key)
                except ServiceException as e:
                    error = ErrorDTO(e.error_code, e.message)
                    return jsonify(error.to_dict()), 403
                except Exception as e:
                    return jsonify(e), 500

                if not valid:
                    ex = ServiceException(ServiceErrorCode.Unauthorized, f"API-Key invalid")
                    error = ErrorDTO(ex.error_code, ex.message)
                    return jsonify(error.to_dict()), 401

                return await f(*args, **kwargs)

            if token is None:
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Token not set")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 401

            if cls._auth_users is None or cls._auth is None:
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Authorize is not initialized")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 401

            if not cls._auth.verify_login(token):
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Token expired")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 401

            token = cls._auth.decode_token(token)
            if token is None or "email" not in token:
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Token invalid")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 401

            user = cls._auth_users.get_auth_user_by_email(token["email"])
            if user is None:
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Token invalid")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 401

            if role is not None and user.auth_role.value < role.value:
                ex = ServiceException(ServiceErrorCode.Unauthorized, f"Role {role} required")
                error = ErrorDTO(ex.error_code, ex.message)
                return jsonify(error.to_dict()), 403

            return await f(*args, **kwargs)

        return decorator

    @classmethod
    def route(cls, path=None, **kwargs):
        # simple decorator for class based views
        def inner(fn):
            cross_origin(fn)
            cls.registered_routes[path] = (fn, kwargs)
            return fn

        return inner

    @classmethod
    def get(cls, path=None, **kwargs):
        return cls.route(path, methods=["GET"], **kwargs)

    @classmethod
    def post(cls, path=None, **kwargs):
        return cls.route(path, methods=["POST"], **kwargs)

    @classmethod
    def head(cls, path=None, **kwargs):
        return cls.route(path, methods=["HEAD"], **kwargs)

    @classmethod
    def put(cls, path=None, **kwargs):
        return cls.route(path, methods=["PUT"], **kwargs)

    @classmethod
    def delete(cls, path=None, **kwargs):
        return cls.route(path, methods=["DELETE"], **kwargs)
