from typing import Optional

from cpl_core.database.context import DatabaseContextABC
from cpl_query.extension import List

from bot_core.logging.database_logger import DatabaseLogger
from bot_data.abc.client_repository_abc import ClientRepositoryABC
from bot_data.abc.server_repository_abc import ServerRepositoryABC
from bot_data.model.client import Client


class ClientRepositoryService(ClientRepositoryABC):
    def __init__(
        self,
        logger: DatabaseLogger,
        db_context: DatabaseContextABC,
        servers: ServerRepositoryABC,
    ):
        self._logger = logger
        self._context = db_context

        self._servers = servers

        ClientRepositoryABC.__init__(self)

    def get_clients(self) -> List[Client]:
        clients = List(Client)
        self._logger.trace(__name__, f"Send SQL command: {Client.get_select_all_string()}")
        results = self._context.select(Client.get_select_all_string())
        for result in results:
            self._logger.trace(__name__, f"Get client with id {result[0]}")
            clients.append(
                Client(
                    result[1],
                    result[2],
                    result[3],
                    result[4],
                    result[5],
                    result[6],
                    self._servers.get_server_by_id(result[7]),
                    result[8],
                    result[9],
                    id=result[0],
                )
            )

        return clients

    def get_client_by_id(self, client_id: int) -> Client:
        self._logger.trace(__name__, f"Send SQL command: {Client.get_select_by_id_string(client_id)}")
        result = self._context.select(Client.get_select_by_id_string(client_id))
        return Client(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[7]),
            result[8],
            result[9],
            id=result[0],
        )

    def get_clients_by_server_id(self, server_id: int) -> List[Client]:
        clients = List(Client)
        self._logger.trace(
            __name__,
            f"Send SQL command: {Client.get_select_by_server_id_string(server_id)}",
        )
        results = self._context.select(Client.get_select_by_server_id_string(server_id))
        for result in results:
            clients.append(
                Client(
                    result[1],
                    result[2],
                    result[3],
                    result[4],
                    result[5],
                    result[6],
                    self._servers.get_server_by_id(result[7]),
                    result[8],
                    result[9],
                    id=result[0],
                )
            )

        return clients

    def get_client_by_discord_id(self, discord_id: int) -> Client:
        self._logger.trace(
            __name__,
            f"Send SQL command: {Client.get_select_by_discord_id_string(discord_id)}",
        )
        result = self._context.select(Client.get_select_by_discord_id_string(discord_id))[0]
        return Client(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[7]),
            result[8],
            result[9],
            id=result[0],
        )

    def find_client_by_discord_id(self, discord_id: int) -> Optional[Client]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {Client.get_select_by_discord_id_string(discord_id)}",
        )
        result = self._context.select(Client.get_select_by_discord_id_string(discord_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return Client(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[7]),
            result[8],
            result[9],
            id=result[0],
        )

    def find_client_by_server_id(self, discord_id: int) -> Optional[Client]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {Client.get_select_by_server_id_string(discord_id)}",
        )
        result = self._context.select(Client.get_select_by_server_id_string(discord_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return Client(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[7]),
            result[8],
            result[9],
            id=result[0],
        )

    def find_client_by_discord_id_and_server_id(self, discord_id: int, server_id: int) -> Optional[Client]:
        self._logger.trace(
            __name__,
            f"Send SQL command: {Client.get_select_by_discord_id_and_server_id_string(discord_id, server_id)}",
        )
        result = self._context.select(Client.get_select_by_discord_id_and_server_id_string(discord_id, server_id))
        if result is None or len(result) == 0:
            return None

        result = result[0]

        return Client(
            result[1],
            result[2],
            result[3],
            result[4],
            result[5],
            result[6],
            self._servers.get_server_by_id(result[7]),
            result[8],
            result[9],
            id=result[0],
        )

    def add_client(self, client: Client):
        self._logger.trace(__name__, f"Send SQL command: {client.insert_string}")
        self._context.cursor.execute(client.insert_string)

    def update_client(self, client: Client):
        self._logger.trace(__name__, f"Send SQL command: {client.udpate_string}")
        self._context.cursor.execute(client.udpate_string)

    def delete_client(self, client: Client):
        self._logger.trace(__name__, f"Send SQL command: {client.delete_string}")
        self._context.cursor.execute(client.delete_string)

    def _get_client_and_server(self, id: int, server_id: int) -> Client:
        server = self._servers.find_server_by_discord_id(server_id)
        if server is None:
            self._logger.warn(__name__, f"Cannot find server by id {server_id}")
            raise Exception("Value not found")

        client = self.find_client_by_discord_id_and_server_id(id, server.id)
        if client is None:
            self._logger.warn(__name__, f"Cannot find client by ids {id}@{server.id}")
            raise Exception("Value not found")

        return client

    def append_sent_message_count(self, client_id: int, server_id: int, value: int):
        client = self._get_client_and_server(client_id, server_id)
        client.sent_message_count += value
        self.update_client(client)

    def append_received_message_count(self, client_id: int, server_id: int, value: int):
        client = self._get_client_and_server(client_id, server_id)
        client.received_message_count += value
        self.update_client(client)

    def append_deleted_message_count(self, client_id: int, server_id: int, value: int):
        client = self._get_client_and_server(client_id, server_id)
        client.deleted_message_count += value
        self.update_client(client)

    def append_received_command_count(self, client_id: int, server_id: int, value: int):
        client = self._get_client_and_server(client_id, server_id)
        client.received_command_count += value
        self.update_client(client)

    def append_moved_users_count(self, client_id: int, server_id: int, value: int):
        client = self._get_client_and_server(client_id, server_id)
        client.moved_users_count += value
        self.update_client(client)
