import { Component, OnInit } from "@angular/core";
import { FormBuilder, FormGroup, Validators } from "@angular/forms";
import { ActivatedRoute, Router } from "@angular/router";
import { TranslateService } from "@ngx-translate/core";
import { catchError } from "rxjs/operators";
import { ResetPasswordDTO } from "src/app/models/auth/reset-password.dto";
import { AuthService } from "src/app/services/auth/auth.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { throwError } from "rxjs";

@Component({
  selector: "app-forget-password",
  templateUrl: "./forget-password.component.html",
  styleUrls: ["./forget-password.component.scss"]
})
export class ForgetPasswordComponent implements OnInit {

  emailForm!: FormGroup;
  passwordForm!: FormGroup;
  submitted = false;
  ready = false;
  repeatErrors = {
    email: false,
    password: false
  };

  resetPasswordId: string | null = null;

  constructor(
    private authService: AuthService,
    private formBuilder: FormBuilder,
    private router: Router,
    private spinnerService: SpinnerService,
    private route: ActivatedRoute,
    private toastService: ToastService,
    private translate: TranslateService
  ) {
  }

  ngOnInit(): void {
    this.spinnerService.showSpinner();
    if (this.authService.isLoggedIn$.value) {
      this.router.navigate(["/dashboard"]);
    }

    this.initForms();
    this.checkResetPasswordId();
    this.spinnerService.hideSpinner();
  }

  initForms(): void {
    this.emailForm = this.formBuilder.group({
      email: [null, [Validators.required, Validators.email]]
    });

    this.passwordForm = this.formBuilder.group({
      password: [null, [Validators.required, Validators.minLength(8)]],
      passwordRepeat: [null, [Validators.required, Validators.minLength(8)]]
    });
  }

  login(): void {
    this.router.navigate(["/auth/login"]);
  }

  register(): void {
    this.router.navigate(["/auth/register"]);
  }

  forgotPassword(): void {
    this.submitted = true;

    if (this.emailForm.invalid) {
      return;
    }

    this.spinnerService.showSpinner();
    this.authService.forgotPassword(this.emailForm.value.email)
      .pipe(catchError(error => {
        this.spinnerService.hideSpinner();
        return throwError(() => error);
      })).subscribe(res => {
      this.spinnerService.hideSpinner();
      this.ready = true;
      setTimeout(() => {
        this.router.navigate(["/dashboard"]);
      }, 5000);
    });
  }

  checkResetPasswordId(): void {
    const id = this.route.snapshot.params["id"];
    if (id) {
      this.resetPasswordId = id;
      this.spinnerService.showSpinner();
      this.authService.getEMailFromforgotPasswordId(id)
        .pipe(catchError(error => {
          this.spinnerService.hideSpinner();
          this.router.navigate(["/auth/forgot-password"]);
          return throwError(() => error);
        })).subscribe(email => {
        this.spinnerService.hideSpinner();
        if (email) {
          this.emailForm.value.email = email;
        } else {
          this.router.navigate(["/auth/forgot-password"]);
        }
      });
    }
  }

  resetPassword(): void {
    const id = this.route.snapshot.params["id"];
    if (this.emailForm.value.password !== this.emailForm.value.passwordRepeat) {
      this.repeatErrors.password = true;
      return;
    }
    this.spinnerService.showSpinner();

    const resetPasswordDTO: ResetPasswordDTO = {
      id,
      password: this.passwordForm.value.password
    };

    this.authService.resetPassword(resetPasswordDTO)
      .pipe(catchError(error => {
        this.router.navigate(["/auth/login"]);
        this.spinnerService.hideSpinner();
        return throwError(() => error);
      }))
      .subscribe(resp => {
        this.spinnerService.hideSpinner();
        this.toastService.success(this.translate.instant("auth.forgot_password.message.reset_password"), this.translate.instant("auth.forgot_password.message.reset_password_d"));
        this.router.navigate(["/auth/login"]);
      });
  }
}
