import { Component, OnInit } from "@angular/core";
import { FormBuilder } from "@angular/forms";
import { TranslateService } from "@ngx-translate/core";
import { catchError } from "rxjs/operators";
import { GuiService } from "src/app/services/gui/gui.service";
import { SettingsService } from "src/app/services/settings/settings.service";
import { SpinnerService } from "src/app/services/spinner/spinner.service";
import { ToastService } from "src/app/services/toast/toast.service";
import { forkJoin, throwError } from "rxjs";
import { Query, ServerConfigQuery, SingleDiscordQuery } from "../../../../../../models/graphql/query.model";
import { Queries } from "../../../../../../models/graphql/queries.model";
import { DataService } from "../../../../../../services/data/data.service";
import { ServerConfigMutationResult } from "../../../../../../models/graphql/result.model";
import { Mutations } from "../../../../../../models/graphql/mutations.model";
import { AuthService } from "../../../../../../services/auth/auth.service";
import { ServerConfig } from "../../../../../../models/config/server-config.model";
import { Server } from "../../../../../../models/data/server.model";
import { ActivatedRoute } from "@angular/router";
import { Channel, ChannelType, Role } from "../../../../../../models/data/discord.model";

type AFKChannelId = {
  id: number;
  value: string;
};

@Component({
  selector: "app-config",
  templateUrl: "./config.component.html",
  styleUrls: ["./config.component.scss"]
})
export class ConfigComponent implements OnInit {
  config: ServerConfig = {
    messageDeleteTimer: 0,
    featureFlags: [],
    afkChannelIds: [],
    moderatorRoleIds: [],
    adminRoleIds: []
  };


  afkChannelIds: AFKChannelId[] = [];
  clonedAfkChannelIds: { [s: number]: AFKChannelId } = {};

  private server: Server = {};
  isTechnician: boolean = false;
  roles?: Role[];
  voiceChannels: Channel[] = [];
  textChannels: Channel[] = [];
  boolAsStrings = [
    { label: this.translate.instant("common.bool_as_string.true"), value: true },
    { label: this.translate.instant("common.bool_as_string.false"), value: false }
  ];

  constructor(
    private data: DataService,
    private settingsService: SettingsService,
    private spinnerService: SpinnerService,
    private guiService: GuiService,
    private formBuilder: FormBuilder,
    private toastService: ToastService,
    private translate: TranslateService,
    private authService: AuthService,
    private spinner: SpinnerService,
    private route: ActivatedRoute
  ) {
  }

  ngOnInit(): void {
    this.spinnerService.showSpinner();

    this.data.getServerFromRoute(this.route).then(async server => {
      this.server = server;
      this.loadConfig();
      let authUser = await this.authService.getLoggedInUser();
      let isUserTechnicianList = authUser?.users?.map(u => u.isTechnician).filter(u => u) ?? [];
      this.isTechnician = isUserTechnicianList.length > 0;
    });
  }

  loadConfig() {
    forkJoin([
      this.data.query<ServerConfigQuery>(Queries.serverConfigQuery, {
          serverId: this.server.id
        },
        (data: Query) => {
          return data.servers[0];
        }),
      this.data.query<SingleDiscordQuery>(Queries.serverConfigDiscordQuery)
    ]).subscribe(data => {
      this.config = data[0].config;
      let id = 0;
      for (const afkChannelId of this.config.afkChannelIds ?? []) {
        this.afkChannelIds.push({ id: id, value: afkChannelId });
        id++;
      }

      if (data[1].discord.guilds) {
        const guild = data[1].discord.guilds.filter(g => g.id === this.server.discordId)[0];
        this.roles = guild.roles ?? undefined;
        this.voiceChannels = guild.channels.filter(x => x.type == ChannelType.voice) ?? undefined;
        this.textChannels = guild.channels.filter(x => x.type == ChannelType.text) ?? undefined;
      }

      this.spinnerService.hideSpinner();
    });
  }

  saveServerConfig() {
    this.spinner.showSpinner();
    this.data.mutation<ServerConfigMutationResult>(Mutations.updateServerConfig, {
        id: this.config.id,
        messageDeleteTimer: this.config.messageDeleteTimer,
        notificationChatId: this.config.notificationChatId,
        maxVoiceStateHours: this.config.maxVoiceStateHours,
        xpPerMessage: this.config.xpPerMessage,
        xpPerReaction: this.config.xpPerReaction,
        maxMessageXpPerHour: this.config.maxMessageXpPerHour,
        xpPerOntimeHour: this.config.xpPerOntimeHour,
        xpPerEventParticipation: this.config.xpPerEventParticipation,
        xpPerAchievement: this.config.xpPerAchievement,
        xpForBirthday: this.config.xpForBirthday,
        afkCommandChannelId: this.config.afkCommandChannelId,
        helpVoiceChannelId: this.config.helpVoiceChannelId,
        teamChannelId: this.config.teamChannelId,
        loginMessageChannelId: this.config.loginMessageChannelId,
        defaultRoleId: this.config.defaultRoleId,
        shortRoleNameOnlySetHighestRole: this.config.shortRoleNameOnlySetHighestRole,
        gameOfferNotificationChatId: this.config.gameOfferNotificationChatId,
        featureFlags: this.config.featureFlags,
        afkChannelIds: this.config.afkChannelIds,
        moderatorRoleIds: this.config.moderatorRoleIds,
        adminRoleIds: this.config.adminRoleIds
      }
    ).pipe(catchError(err => {
      this.spinner.hideSpinner();
      return throwError(err);
    })).subscribe(result => {
      this.spinner.hideSpinner();
      this.toastService.success(this.translate.instant("view.server.config.message.server_config_create"), this.translate.instant("view.server.config.message.server_config_create_d"));
    });
  }
}
